import React, { useState } from 'react';
import styled from 'styled-components';
import Loading from '../../util/Loading';
import GuessPane from "./GuessPane";
import PositionedStreetView from "./PositionedStreetView";
import { dispatch, useCurrentRound, useTargetPoint } from '../../../domain/gameStore';
import usePreventNavigation from '../../../hooks/usePreventNavigation';

const Container = styled.div`
  height: 100%;
  width: 100%;
  display: flex;
  flex-flow: column nowrap;
  justify-content: space-between;
  align-items: center;

  @media only screen and (min-width: 600px) and (min-height: 600px) {
    display: block;
    position: relative;
    margin-bottom: 2px;
    margin-right: 2px;
  }
`

const StreetViewWrapper = styled.div`
  height: 100%;
  width: 100%;
  margin-bottom: 2px;
  flex: 3;
`

export default () => {
  // warn the user if they navigate away
  usePreventNavigation();

  const [submitDisabled, setSubmitDisabled] = useState(false);
  const [selectedPoint, setSelectedPoint] = useState(null);
  const currentRound = useCurrentRound();
  const targetPoint = useTargetPoint();

  if (currentRound === null) {
    dispatch.goToSummary();
    return <Loading/>
  }

  const handleSubmitGuess = async () => {
    setSubmitDisabled(true);
    await dispatch.submitGuess(selectedPoint);
  }

  return (
    <Container>
      <StreetViewWrapper>
        <PositionedStreetView position={targetPoint} resetPosition={targetPoint} />
      </StreetViewWrapper>
      <GuessPane
        onTimeout={handleSubmitGuess}
        onSelectPoint={setSelectedPoint}
        onSubmitGuess={handleSubmitGuess}
        submitDisabled={submitDisabled || selectedPoint === null}
      />
    </Container>
  );
}
