import { PRE_GAME, PRE_ROUND, IN_ROUND, POST_ROUND, POST_GAME } from "./GameState";
import { createStore } from "../store";
import { joinGame, sendGuess, getCurrentRound } from "./apiMethods";
import {
  saveGameInfoToLocalStorage,
  clearGameInfoFromLocalStorage,
  clearPointFromLocalStorage,
  clearTimerFromLocalStorage,
} from "./localStorageMethods";

export const [
  {
    useGameId,
    usePlayerName,
    useLastRound,
    usePlayerId,
    useGameState,
    useCurrentRound,
    useTargetPoint,
    useRoundSeconds,
  },
  dispatch,
] = createStore({
  gameId: null,
  playerName: null,
  lastRound: {
    roundNum: -1,
    targetPoint: {
      lat: 0,
      lng: 0,
    },
    score: -1,
    totalScore: -1,
  },
  playerId: null,
  gameState: PRE_GAME,
  currentRound: null,
  targetPoint: null,
  roundSeconds: 0,
}, {
  setPlayerName: ([set], playerName) => set({ playerName }),
  goToLobby: ([set], gameId) => set({ 
    gameId,
    playerId: null,
    gameState: PRE_ROUND,
  }),
  joinGame: async ([set, get]) => {
    const gameId = get.gameId();
    const name = get.playerName();
    const { playerId } = await joinGame(gameId, name);
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    set({
      playerId,
      currentRound,
      targetPoint: coord,
      roundSeconds: timer,
    });
    clearTimerFromLocalStorage();
    clearPointFromLocalStorage();
    saveGameInfoToLocalStorage(gameId, name, playerId);
  },
  rejoinGame: async ([set], gameId, playerName, playerId) => {
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    set({ 
      gameId, 
      playerName, 
      playerId,
      currentRound,
      targetPoint: coord,
      roundSeconds: timer,
      gameState: IN_ROUND,
    });
  },
  startRound: ([set]) => set({ gameState: IN_ROUND }),
  submitGuess: async ([set, get], selectedPoint) => {
    const gameId = get.gameId();
    const playerId = get.playerId();
    const roundNum = get.currentRound();
    const targetPoint = get.targetPoint();
    const { score, totalScore } = await sendGuess(
      gameId,
      playerId,
      roundNum,
      selectedPoint || { timeout: true }
    );
    const { currentRound, coord, timer } = await getCurrentRound(gameId, playerId);
    set({
      lastRound: {
        roundNum,
        targetPoint,
        score,
        totalScore,
      },
      currentRound,
      targetPoint: coord,
      roundSeconds: timer,
      gameState: POST_ROUND,
    });
    clearTimerFromLocalStorage();
    clearPointFromLocalStorage();
  },
  goToSummary: ([set, get], gameId, clearSavedGame = true) => {
    set({
      gameId: gameId || get.gameId(),
      gameState: POST_GAME,
    });
    if (clearSavedGame) {
      clearTimerFromLocalStorage();
      clearPointFromLocalStorage();
      clearGameInfoFromLocalStorage();
    }
  },
  setRoundSeconds: ([set], roundSeconds) => set({ roundSeconds }),
});